DROP TRIGGER context_insert;
CREATE TRIGGER context_insert
BEFORE INSERT ON context
BEGIN
    SELECT CASE
        WHEN NEW.predefined_id <> 0 AND (SELECT COUNT(*) FROM context WHERE predefined_id = NEW.predefined_id) > 0
            THEN RAISE (IGNORE)
    END;

    SELECT CASE
        WHEN (SELECT id FROM context WHERE id = NEW.parent) IS NULL AND NEW.parent IS NOT NULL
            THEN RAISE (ABORT, 'CONTEXT_PARENT_INSERT_FIRST_VIOLATION')
    END;
END;

-- Extend business object entities
ALTER TABLE context ADD COLUMN deleted INTEGER DEFAULT 0;
ALTER TABLE context_deleted ADD COLUMN parent TEXT DEFAULT NULL;
ALTER TABLE context_deleted ADD COLUMN deleted INTEGER DEFAULT 1;

ALTER TABLE project ADD COLUMN deleted INTEGER DEFAULT 0;
ALTER TABLE project_deleted ADD COLUMN deleted INTEGER DEFAULT 1;

ALTER TABLE task ADD COLUMN deleted INTEGER DEFAULT 0;
ALTER TABLE task_deleted ADD COLUMN deleted INTEGER DEFAULT 1;
ALTER TABLE task_deleted ADD COLUMN start_reminder INTEGER DEFAULT 0;
ALTER TABLE task_deleted ADD COLUMN due_reminder INTEGER DEFAULT 0;
ALTER TABLE task_deleted ADD COLUMN period_type INTEGER DEFAULT 0;
ALTER TABLE task_deleted ADD COLUMN period_interval INTEGER DEFAULT 0;
ALTER TABLE task_deleted ADD COLUMN days INTEGER DEFAULT 0;

UPDATE context SET deleted = 0;
UPDATE project SET deleted = 0;
UPDATE task SET deleted = 0;
UPDATE context_deleted SET deleted = 1;
UPDATE project_deleted SET deleted = 1;
UPDATE task_deleted SET deleted = 1;

INSERT INTO context SELECT * FROM context_deleted;
INSERT INTO project SELECT * FROM project_deleted;
INSERT INTO task SELECT * FROM task_deleted;

-- Simplify synchronization queue
CREATE TABLE sync_queue_tmp ( id INTEGER PRIMARY KEY,
    uuid TEXT NOT NULL,
    action TEXT NOT NULL
);

/* Copy data from the old one */
/*
DROP TRIGGER sync_queue_insert;
CREATE TRIGGER sync_queue_insert
BEFORE INSERT ON sync_queue_tmp
BEGIN
    -- Ignore silently any data duplication
    SELECT CASE
        WHEN (SELECT COUNT(*) FROM sync_queue_tmp) > 0 AND (SELECT COUNT(*) FROM sync_queue_tmp WHERE uuid = NEW.uuid) > 0
            THEN RAISE (IGNORE)
    END;
END;
*/
INSERT INTO sync_queue_tmp SELECT id, uuid, action FROM sync_queue ORDER BY id ASC;

/* Drop old entity */
DROP TABLE sync_queue;

/* Rename new one */
ALTER TABLE sync_queue_tmp RENAME TO sync_queue;
